@@ -2,6 +2,8 @@ use bevy::prelude::*;
 use bevy_asset_loader::{AssetCollection, AssetLoader};
 
 /// This example demonstrates how to load a texture atlas from a sprite sheet
+///
+/// Requires the feature 'render' (part of default features)
 fn main() {
     let mut app = App::build();
     AssetLoader::new(MyStates::AssetLoading, MyStates::Next)
@@ -18,7 +20,8 @@ fn main() {
 
 #[derive(AssetCollection)]
 struct MyAssets {
-    #[asset(texture_atlas(cell_width = 100., cell_height = 96., columns = 8, rows = 1))]
+    // if the sheet would have padding, we could set that with `padding_x` and `padding_y`
+    #[asset(texture_atlas(tile_size_x = 100., tile_size_y = 96., columns = 8, rows = 1))]
     #[asset(path = "textures/female_adventurer.png")]
     female_adventurer: Handle<TextureAtlas>,
 }
